/************************************************************************
|                          EXAR Corporation
|                      Data Communications Division
|         DOS Test Program Source code for XR16L788 & XR16L784
|************************************************************************
|
[Date]           April 02, 2002

[Disclaimer]
 2002 EXAR Corporation
All Rights Reserved.

Exar Corporation ("Exar") hereby grants the User of this Exar product-specific
source code in "C" a non-exclusive, nontransferable license to use this Exar
product-specific source code under the following terms. Before using this Exar
product-specific source code, the User should read the following use
restrictions.  If the User does not accept these terms, the Exar
product-specific source code should not be used or downloaded.

The User is granted this license only to use the Exar product-specific source
code for User's internal use, and is not granted rights to sell, loan,
rent, lease or license the Exar product-specific source code in whole or in
part, or in modified form to anyone other than User.  User may modify the
Exar product-specific source code to suit its specific applications but
rights to derivative works and such modifications shall belong to Exar.

This Exar product-specific source code is provided on an "AS IS" basis and
Exar makes absolutely no warranty with respect to the information contained
herein.  EXAR DISCLAIMS AND USER WAIVES ALL WARRANTIES, EXPRESS OR IMPLIED,
INCLUDING WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
The entire risk as to quality and performance is with the User.
ACCORDINGLY, IN NO EVENT SHALL EXAR BE LIABLE FOR ANY DAMAGES, WHETHER IN
CONTRACT OR TORT, INCLUDING ANY LOST PROFITS OR OTHER INCIDENTAL,
CONSEQUENTIAL, EXEMPLARY, OR PUNITIVE DAMAGES ARISING OUT OF THE USE OR
APPLICATION OF THE EXAR PRODUCT-SPECIFIC SOURCE CODE.  Further, Exar reserves
the right to make changes without notice to improve reliability, function or
design.  Exar does not convey any license under patent rights or any other
intellectual property rights, including those of third parties.

Exar is not obligated to provide maintenance or support for the Exar
product-specific source code.
[Copyright]      Copyright 2002 EXAR Corporation.

*****************************************************************************
*****************************************************************************/

/* This test program does full duplex receive and transmit on one
channel of the XR16L788 or 784 which is connected to a remote UART. It
uses interrupts and trigger levels. Also, the user has the choice
of using different flow control mechanisms and baud rates. The max.
baud rate is 921.6kbps for 14.7MHz clock. */


#include <iostream.h>
#include <math.h>
#include <time.h>
#include <stdio.h>
#include <dos.h>
#include "xr16l788.h"

#define intvect  0x0d		//default interrupt vector for IRQ5
#define ch			0			// choose channel of 788 you want to use
#define	SIZE   	40000		// size of software buffer used to store RX data
#define INTR	0x03			//which interrupts you want enabled in
									//the device 0x03 means TX & RX interrupts

int baseaddr = 0x500;		// I/O base address of the device
long int bufferin, bufferout;
char *buffer;
int r_trg = 48;  				//rx trigger level
int t_trg = 8;   			 	// tx trigger level
int txd = 65 - t_trg; 		//no. of characters to be loaded into the TX FIFO
char data [0x100]; 			// data to be transmitted
int databuf;

/* oldportisr is the existing interrupt service routine */

void interrupt (*oldportisr) (...);

/* myportisr() is the new interrupt service routine
	 which will be used for this program */

void interrupt myportisr(...)
{
 int int_val = read (ch, ISR);
 int i;
 write (ch, IER, 0x00); 		//disable interrupts after reading ISR register

 START:
  switch (int_val) {
  /* RX interrupt */
	case 0xc4	:	for (i = 0; i < r_trg; i++) {
							buffer[bufferin] = read (ch, RHR);
							bufferin++;
							if (bufferin == SIZE) bufferin = 0;
						}
						break;
  /* RX Timeout Interrupt */
	case 0xcc	:	while ( (read(ch, LSR) & 0x01 )  ) {
						buffer[bufferin] = read (ch, RHR);
						bufferin++;
						if (bufferin == SIZE) bufferin = 0;
					 }
					 break;
	/* TX interrupt */
	case 0xc2	:	for (i = 0; i < txd; i++) {
						write (ch, THR, data[databuf]);
						databuf++;
						if (databuf == 0x100) databuf = 0;
					}
					break;
	default		:	  ;			// we do not expect any other interrupt
										// so do nothing & exit
  }

  outportb (0x20, 0x20); 		// this is required for the PIC
  write (ch, IER, INTR);
}

void main (void)
{
  int i, flowcontrol, flag, char0, dll, dlm, hyst;
  int dt0 = 0;
  int error = 0;

  bufferin = 0;
  bufferout = 0;
  databuf = 0;

  buffer = new char[SIZE];

	for (i = 0; i < 0x100; i++) {
		switch (i) {
			case 0x11 : case 0x13 :
			case 0x12 : case 0x14 :
									data[i] = i + 0x80; 			// to prevent sending
																		//flow control chars
									break;
			default	:				data[i] = i;
		}
	}

/* Test to see if 788 or 784 is there by checking the device ID*/
	if ( (config_read (DVID) != 0x28) || (config_read (DVID) != 0x24) ) {
		cout << "\n UART Not detected\n";
		return;
	}

	 oldportisr = getvect(intvect);		// store the current isr
	 setvect(intvect, myportisr);			// myportisr is a pointer to an
													// interrupt function

/* Get DLL values from user : use 0x00 for DLM */
	cout << "\nEnter DLL in hex: 0x";
	cin >> hex >> dll;					// baud rate = (921.6 / dll) kbps
	dlm = 0;

/* Use no hysteresis */
	hyst = 0;

/* Obtain flow control selection from user */
	cout << "\nEnter flow control parameters from the following options\n";
	cout << "\t0) No Flow Control\n";
	cout << "\t1) Hardware Flow Control (Auto-RTS & Auto-CTS)\n";
	cout << "\t2) Software Flow Control, with XON1/XOFF1\n";
	cout << "\t3) Software Flow Control, with XON2/XOFF2\n";
	cout << "\t4) Software Flow Control, with XON1 & XON2/XOFF1 & XOFF2\n";

	cin >> flag;
	switch (flag)	{
		case 0  :	flowcontrol = 0x00; break;
		case 1	:	flowcontrol = 0xc0;	break;
		case 2	:	flowcontrol = 0x0a;	break;
		case 3	:	flowcontrol = 0x05;	break;
		case 4	:	flowcontrol = 0x0f;	break;
		default	:	flowcontrol = 0x00;	break;
	}

/* Initialization */
	config_write (RESET, 0xff);		// reset all channels
	config_write (REG2, 0x01);			// enable simultaneous write to all channels
	write (ch, IER, 0x00);				// disable interrupts
	write (ch, FCR, 0x07);				// enable & flush FIFOs
	write (ch, LCR, 0x80);				// setup baud rate registers
	write (ch, DLL, dll);
	write (ch, DLM, dlm);
	write (ch, LCR, 0x03);				//setting 8N1
	write (ch, FCTR, 0xc0 | hyst);  	//trigger table D selected with hysteresis
	write (ch, RXTRIG, r_trg);   		// RX trigger level - usually 64
	write (ch, TXTRIG, t_trg);  		//TX trigger level - usually 16
	write (ch, EFR, 0x10|flowcontrol);	//enable shaded bits + flow control
	write (ch, XOFF1, 0x11);
	write (ch, XON1, 0x13);
	write (ch, XOFF2, 0x12);
	write (ch, XON2, 0x14);


/* If hardware flow control is used, enable RTS initially */
	if (flag == 1) write (ch, MCR, 0x02);
	else write (ch, MCR, 0x00);

/* disable simultaneous write to all channels */
	config_write (REG2, 0x00);

  clrscr();
  cout << "\nFull-duplex TX/RX in two channels of XR16L788 in progress using ";
  switch (flag) {
	case 0	:	cout << "\nNo Flow Control\n";
				break;
	case 1	:	cout << "\nHardware Flow Control\n";
				break;
	case 2	:	cout << "\nSoftware Flow Control, with XON1/XOFF1\n";
				break;
	case 3	:	cout << "\nSoftware Flow Control, with XON2/XOFF2\n";
				break;
	case 4	:	cout << "\nSoftware Flow Control, with XON1 & XON2/XOFF1 & XOFF2\n";
				break;
  }

/* enable IRQ5 in the PIC & enable UART interrupts */
	outportb (0x21, ( inportb(0x21) & 0xdf) );
	write (ch, IER, INTR);


	while ( 1 ) {
		if (bufferin != bufferout) {
			char0 = ( (int)buffer[bufferout]) & 0xff ;
			if (buffer[bufferout] != data[dt0]) {
				cout << "\nError: Expected 0x" <<  hex << ((int)data[dt0] & 0xff)\
				 << "; Received 0x" << hex << char0;
				dt0++;
				error++;
				cout << "\nBufferin: " << dec << bufferin << "; Bufferout: " << dec << bufferout;
			}
			bufferout++;
			if (bufferout == SIZE) {
				bufferout = 0;
			}
			dt0++;
			if (dt0 == 0x100) dt0 = 0;
		}
		if (error == 5) goto END; 			// stop test after 5 RX errors
  }

END:

	delete[] buffer;
	write (ch, IER, 0x00);
	outportb (0x21, ( inportb (0x21) | 0x20 ) );
	setvect (intvect, oldportisr);
	config_write (RESET, 0xff);
return;
}
